@@ -15,16 +15,13 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use std::{
-    iter::{repeat, repeat_with},
-    sync::Arc,
-};
+use std::{hint, iter::repeat_with, sync::Arc};
 
 use arrow_array::{Array, ArrayRef, Int32Array, UnionArray};
 use arrow_buffer::{NullBuffer, ScalarBuffer};
 use arrow_schema::{DataType, Field, UnionFields};
 use criterion::*;
-use rand::{rng, Rng};
+use rand::{Rng, rng};
 
 fn array_with_nulls() -> ArrayRef {
     let mut rng = rng();
@@ -66,14 +63,13 @@ fn criterion_benchmark(c: &mut Criterion) {
                         fields,
                         type_ids.cycle().take(4096).collect(),
                         None,
-                        repeat(array_with_nulls())
-                            .take(with_nulls as usize)
-                            .chain(repeat(array_without_nulls()).take(without_nulls as usize))
+                        std::iter::repeat_n(array_with_nulls(), with_nulls as usize)
+                            .chain(std::iter::repeat_n(array_without_nulls(), without_nulls as usize))
                             .collect(),
                     )
                     .unwrap();
 
-                    b.iter(|| black_box(array.logical_nulls()))
+                    b.iter(|| hint::black_box(array.logical_nulls()))
                 },
             );
         }
