@@ -16,7 +16,7 @@
 // under the License.
 #![allow(clippy::enum_clike_unportable_variant)]
 
-use crate::{make_array, Array, ArrayRef};
+use crate::{Array, ArrayRef, make_array};
 use arrow_buffer::bit_chunk_iterator::{BitChunkIterator, BitChunks};
 use arrow_buffer::buffer::NullBuffer;
 use arrow_buffer::{BooleanBuffer, MutableBuffer, ScalarBuffer};
@@ -137,11 +137,11 @@ impl UnionArray {
     ///
     /// # Safety
     ///
-    /// The `type_ids` values should be positive and must match one of the type ids of the fields provided in `fields`.
+    /// The `type_ids` values should be non-negative and must match one of the type ids of the fields provided in `fields`.
     /// These values are used to index into the `children` arrays.
     ///
     /// The `offsets` is provided in the case of a dense union, sparse unions should use `None`.
-    /// If provided the `offsets` values should be positive and must be less than the length of the
+    /// If provided the `offsets` values should be non-negative and must be less than the length of the
     /// corresponding array.
     ///
     /// In both cases above we use signed integer types to maintain compatibility with other
@@ -165,8 +165,8 @@ impl UnionArray {
             .len(len);
 
         let data = match offsets {
-            Some(offsets) => builder.add_buffer(offsets.into_inner()).build_unchecked(),
-            None => builder.build_unchecked(),
+            Some(offsets) => unsafe { builder.add_buffer(offsets.into_inner()).build_unchecked() },
+            None => unsafe { builder.build_unchecked() },
         };
         Self::from(data)
     }
@@ -219,7 +219,7 @@ impl UnionArray {
                 _ => {
                     return Err(ArrowError::InvalidArgumentError(
                         "Type Ids values must match one of the field type ids".to_owned(),
-                    ))
+                    ));
                 }
             }
         }
@@ -230,7 +230,7 @@ impl UnionArray {
             if iter.any(|(type_id, &offset)| offset < 0 || offset >= array_lens[*type_id as usize])
             {
                 return Err(ArrowError::InvalidArgumentError(
-                    "Offsets must be positive and within the length of the Array".to_owned(),
+                    "Offsets must be non-negative and within the length of the Array".to_owned(),
                 ));
             }
         }
@@ -312,7 +312,7 @@ impl UnionArray {
     }
 
     /// Returns whether the `UnionArray` is dense (or sparse if `false`).
-    fn is_dense(&self) -> bool {
+    pub fn is_dense(&self) -> bool {
         match self.data_type() {
             DataType::Union(_, mode) => mode == &UnionMode::Dense,
             _ => unreachable!("Union array's data type is not a union!"),
@@ -1877,7 +1877,7 @@ mod tests {
 
         assert_eq!(
             err.to_string(),
-            "Invalid argument error: Offsets must be positive and within the length of the Array"
+            "Invalid argument error: Offsets must be non-negative and within the length of the Array"
         );
 
         let offsets = Some(vec![0, 1].into());
