@@ -15,7 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::builder::{ArrayBuilder, UInt8BufferBuilder};
+use crate::builder::ArrayBuilder;
 use crate::{ArrayRef, FixedSizeBinaryArray};
 use arrow_buffer::Buffer;
 use arrow_buffer::NullBufferBuilder;
@@ -42,7 +42,7 @@ use std::sync::Arc;
 /// ```
 #[derive(Debug)]
 pub struct FixedSizeBinaryBuilder {
-    values_builder: UInt8BufferBuilder,
+    values_builder: Vec<u8>,
     null_buffer_builder: NullBufferBuilder,
     value_length: i32,
 }
@@ -61,7 +61,7 @@ impl FixedSizeBinaryBuilder {
             "value length ({byte_width}) of the array must >= 0"
         );
         Self {
-            values_builder: UInt8BufferBuilder::new(capacity * byte_width as usize),
+            values_builder: Vec::with_capacity(capacity * byte_width as usize),
             null_buffer_builder: NullBufferBuilder::new(capacity),
             value_length: byte_width,
         }
@@ -79,7 +79,7 @@ impl FixedSizeBinaryBuilder {
                     .to_string(),
             ))
         } else {
-            self.values_builder.append_slice(value.as_ref());
+            self.values_builder.extend_from_slice(value.as_ref());
             self.null_buffer_builder.append_non_null();
             Ok(())
         }
@@ -89,7 +89,7 @@ impl FixedSizeBinaryBuilder {
     #[inline]
     pub fn append_null(&mut self) {
         self.values_builder
-            .append_slice(&vec![0u8; self.value_length as usize][..]);
+            .extend(std::iter::repeat_n(0u8, self.value_length as usize));
         self.null_buffer_builder.append_null();
     }
 
@@ -97,7 +97,7 @@ impl FixedSizeBinaryBuilder {
     #[inline]
     pub fn append_nulls(&mut self, n: usize) {
         self.values_builder
-            .append_slice(&vec![0u8; self.value_length as usize * n][..]);
+            .extend(std::iter::repeat_n(0u8, self.value_length as usize * n));
         self.null_buffer_builder.append_n_nulls(n);
     }
 
@@ -110,7 +110,7 @@ impl FixedSizeBinaryBuilder {
     pub fn finish(&mut self) -> FixedSizeBinaryArray {
         let array_length = self.len();
         let array_data_builder = ArrayData::builder(DataType::FixedSizeBinary(self.value_length))
-            .add_buffer(self.values_builder.finish())
+            .add_buffer(std::mem::take(&mut self.values_builder).into())
             .nulls(self.null_buffer_builder.finish())
             .len(array_length);
         let array_data = unsafe { array_data_builder.build_unchecked() };
