@@ -22,8 +22,8 @@ use std::sync::Arc;
 use arrow_buffer::{Buffer, NullBufferBuilder, ScalarBuffer};
 use arrow_data::{ByteView, MAX_INLINE_VIEW_LEN};
 use arrow_schema::ArrowError;
-use hashbrown::hash_table::Entry;
 use hashbrown::HashTable;
+use hashbrown::hash_table::Entry;
 
 use crate::builder::ArrayBuilder;
 use crate::types::bytes::ByteArrayNativeType;
@@ -121,7 +121,7 @@ impl<T: ByteViewType + ?Sized> GenericByteViewBuilder<T> {
     /// growing buffer size exponentially from 8KB up to 2MB. The
     /// first buffer allocated is 8KB, then 16KB, then 32KB, etc up to 2MB.
     ///
-    /// If this method is used, any new buffers allocated are  
+    /// If this method is used, any new buffers allocated are
     /// exactly this size. This can be useful for advanced users
     /// that want to control the memory usage and buffer count.
     ///
@@ -188,10 +188,10 @@ impl<T: ByteViewType + ?Sized> GenericByteViewBuilder<T> {
     /// (2) The range `offset..offset+length` must be within the bounds of the block
     /// (3) The data in the block must be valid of type `T`
     pub unsafe fn append_view_unchecked(&mut self, block: u32, offset: u32, len: u32) {
-        let b = self.completed.get_unchecked(block as usize);
+        let b = unsafe { self.completed.get_unchecked(block as usize) };
         let start = offset as usize;
         let end = start.saturating_add(len as usize);
-        let b = b.get_unchecked(start..end);
+        let b = unsafe { b.get_unchecked(start..end) };
 
         let view = make_view(b, block, offset);
         self.views_buffer.push(view);
@@ -306,15 +306,30 @@ impl<T: ByteViewType + ?Sized> GenericByteViewBuilder<T> {
     /// - String length exceeds `u32::MAX`
     #[inline]
     pub fn append_value(&mut self, value: impl AsRef<T::Native>) {
+        self.try_append_value(value).unwrap()
+    }
+
+    /// Appends a value into the builder
+    ///
+    /// # Errors
+    ///
+    /// Returns an error if:
+    /// - String buffer count exceeds `u32::MAX`
+    /// - String length exceeds `u32::MAX`
+    #[inline]
+    pub fn try_append_value(&mut self, value: impl AsRef<T::Native>) -> Result<(), ArrowError> {
         let v: &[u8] = value.as_ref().as_ref();
-        let length: u32 = v.len().try_into().unwrap();
+        let length: u32 = v.len().try_into().map_err(|_| {
+            ArrowError::InvalidArgumentError(format!("String length {} exceeds u32::MAX", v.len()))
+        })?;
+
         if length <= MAX_INLINE_VIEW_LEN {
             let mut view_buffer = [0; 16];
             view_buffer[0..4].copy_from_slice(&length.to_le_bytes());
             view_buffer[4..4 + v.len()].copy_from_slice(v);
             self.views_buffer.push(u128::from_le_bytes(view_buffer));
             self.null_buffer_builder.append_non_null();
-            return;
+            return Ok(());
         }
 
         // Deduplication if:
@@ -339,7 +354,7 @@ impl<T: ByteViewType + ?Sized> GenericByteViewBuilder<T> {
                     self.views_buffer.push(self.views_buffer[*idx]);
                     self.null_buffer_builder.append_non_null();
                     self.string_tracker = Some((ht, hasher));
-                    return;
+                    return Ok(());
                 }
                 Entry::Vacant(vacant) => {
                     // o.w. we insert the (string hash -> view index)
@@ -356,17 +371,28 @@ impl<T: ByteViewType + ?Sized> GenericByteViewBuilder<T> {
             let to_reserve = v.len().max(self.block_size.next_size() as usize);
             self.in_progress.reserve(to_reserve);
         };
+
         let offset = self.in_progress.len() as u32;
         self.in_progress.extend_from_slice(v);
 
+        let buffer_index: u32 = self.completed.len().try_into().map_err(|_| {
+            ArrowError::InvalidArgumentError(format!(
+                "Buffer count {} exceeds u32::MAX",
+                self.completed.len()
+            ))
+        })?;
+
         let view = ByteView {
             length,
+            // This won't panic as we checked the length of prefix earlier.
             prefix: u32::from_le_bytes(v[0..4].try_into().unwrap()),
-            buffer_index: self.completed.len() as u32,
+            buffer_index,
             offset,
         };
         self.views_buffer.push(view.into());
         self.null_buffer_builder.append_non_null();
+
+        Ok(())
     }
 
     /// Append an `Option` value into the builder
@@ -390,7 +416,7 @@ impl<T: ByteViewType + ?Sized> GenericByteViewBuilder<T> {
         self.flush_in_progress();
         let completed = std::mem::take(&mut self.completed);
         let nulls = self.null_buffer_builder.finish();
-        if let Some((ref mut ht, _)) = self.string_tracker.as_mut() {
+        if let Some((ht, _)) = self.string_tracker.as_mut() {
             ht.clear();
         }
         let views = std::mem::take(&mut self.views_buffer);
@@ -581,7 +607,6 @@ mod tests {
     use core::str;
 
     use super::*;
-    use crate::Array;
 
     #[test]
     fn test_string_view_deduplicate() {
@@ -688,7 +713,10 @@ mod tests {
         );
 
         let err = v.try_append_view(0, u32::MAX, 1).unwrap_err();
-        assert_eq!(err.to_string(), "Invalid argument error: Range 4294967295..4294967296 out of bounds for block of length 17");
+        assert_eq!(
+            err.to_string(),
+            "Invalid argument error: Range 4294967295..4294967296 out of bounds for block of length 17"
+        );
 
         let err = v.try_append_view(0, 1, u32::MAX).unwrap_err();
         assert_eq!(
@@ -739,10 +767,12 @@ mod tests {
             assert_eq!(fixed_builder.completed.len(), 2_usize.pow(i + 1) - 1);
 
             // Every buffer is fixed size
-            assert!(fixed_builder
-                .completed
-                .iter()
-                .all(|b| b.len() == STARTING_BLOCK_SIZE as usize));
+            assert!(
+                fixed_builder
+                    .completed
+                    .iter()
+                    .all(|b| b.len() == STARTING_BLOCK_SIZE as usize)
+            );
         }
 
         // Add one more value, and the buffer stop growing.
