@@ -27,7 +27,7 @@ use std::fmt::Display;
 use comfy_table::{Cell, Table};
 
 use arrow_array::{Array, ArrayRef, RecordBatch};
-use arrow_schema::ArrowError;
+use arrow_schema::{ArrowError, SchemaRef};
 
 use crate::display::{ArrayFormatter, FormatOptions};
 
@@ -60,11 +60,43 @@ use crate::display::{ArrayFormatter, FormatOptions};
 /// | 5 | e |
 /// +---+---+"#);
 /// ```
-pub fn pretty_format_batches(results: &[RecordBatch]) -> Result<impl Display, ArrowError> {
+pub fn pretty_format_batches(results: &[RecordBatch]) -> Result<impl Display + use<>, ArrowError> {
     let options = FormatOptions::default().with_display_error(true);
     pretty_format_batches_with_options(results, &options)
 }
 
+/// Create a visual representation of [`RecordBatch`]es with a provided schema.
+///
+/// Useful to display empty batches.
+///
+/// # Example
+/// ```
+/// # use std::sync::Arc;
+/// # use arrow_array::{ArrayRef, Int32Array, RecordBatch, StringArray};
+/// # use arrow_cast::pretty::pretty_format_batches_with_schema;
+/// # use arrow_schema::{DataType, Field, Schema};
+/// let schema = Arc::new(Schema::new(vec![
+///     Field::new("a", DataType::Int32, false),
+///     Field::new("b", DataType::Utf8, true),
+/// ]));
+/// // Note, returned object implements `Display`
+/// let pretty_table = pretty_format_batches_with_schema(schema, &[]).unwrap();
+/// let table_str = format!("Batches:\n{pretty_table}");
+/// assert_eq!(table_str,
+/// r#"Batches:
+/// +---+---+
+/// | a | b |
+/// +---+---+
+/// +---+---+"#);
+/// ```
+pub fn pretty_format_batches_with_schema(
+    schema: SchemaRef,
+    results: &[RecordBatch],
+) -> Result<impl Display + use<>, ArrowError> {
+    let options = FormatOptions::default().with_display_error(true);
+    create_table(Some(schema), results, &options)
+}
+
 /// Create a visual representation of [`RecordBatch`]es with formatting options.
 ///
 /// # Arguments
@@ -98,8 +130,8 @@ pub fn pretty_format_batches(results: &[RecordBatch]) -> Result<impl Display, Ar
 pub fn pretty_format_batches_with_options(
     results: &[RecordBatch],
     options: &FormatOptions,
-) -> Result<impl Display, ArrowError> {
-    create_table(results, options)
+) -> Result<impl Display + use<>, ArrowError> {
+    create_table(None, results, options)
 }
 
 /// Create a visual representation of [`ArrayRef`]
@@ -110,7 +142,7 @@ pub fn pretty_format_batches_with_options(
 pub fn pretty_format_columns(
     col_name: &str,
     results: &[ArrayRef],
-) -> Result<impl Display, ArrowError> {
+) -> Result<impl Display + use<>, ArrowError> {
     let options = FormatOptions::default().with_display_error(true);
     pretty_format_columns_with_options(col_name, results, &options)
 }
@@ -122,7 +154,7 @@ pub fn pretty_format_columns_with_options(
     col_name: &str,
     results: &[ArrayRef],
     options: &FormatOptions,
-) -> Result<impl Display, ArrowError> {
+) -> Result<impl Display + use<>, ArrowError> {
     create_column(col_name, results, options)
 }
 
@@ -139,30 +171,42 @@ pub fn print_columns(col_name: &str, results: &[ArrayRef]) -> Result<(), ArrowEr
 }
 
 /// Convert a series of record batches into a table
-fn create_table(results: &[RecordBatch], options: &FormatOptions) -> Result<Table, ArrowError> {
+fn create_table(
+    schema_opt: Option<SchemaRef>,
+    results: &[RecordBatch],
+    options: &FormatOptions,
+) -> Result<Table, ArrowError> {
     let mut table = Table::new();
     table.load_preset("||--+-++|    ++++++");
 
+    let schema_opt = schema_opt.or_else(|| {
+        if results.is_empty() {
+            None
+        } else {
+            Some(results[0].schema())
+        }
+    });
+
+    if let Some(schema) = schema_opt {
+        let mut header = Vec::new();
+        for field in schema.fields() {
+            if options.types_info() {
+                header.push(Cell::new(format!(
+                    "{}\n{}",
+                    field.name(),
+                    field.data_type()
+                )))
+            } else {
+                header.push(Cell::new(field.name()));
+            }
+        }
+        table.set_header(header);
+    }
+
     if results.is_empty() {
         return Ok(table);
     }
 
-    let schema = results[0].schema();
-
-    let mut header = Vec::new();
-    for field in schema.fields() {
-        if options.types_info() {
-            header.push(Cell::new(format!(
-                "{}\n{}",
-                field.name(),
-                field.data_type()
-            )))
-        } else {
-            header.push(Cell::new(field.name()));
-        }
-    }
-    table.set_header(header);
-
     for batch in results {
         let formatters = batch
             .columns()
@@ -221,7 +265,7 @@ mod tests {
     use arrow_buffer::{IntervalDayTime, IntervalMonthDayNano, ScalarBuffer};
     use arrow_schema::*;
 
-    use crate::display::array_value_to_string;
+    use crate::display::{DurationFormat, array_value_to_string};
 
     use super::*;
 
@@ -1186,4 +1230,57 @@ mod tests {
         let actual: Vec<&str> = batch.lines().collect();
         assert_eq!(expected_table, actual, "Actual result:\n{batch}");
     }
+
+    #[test]
+    fn duration_pretty_and_iso_extremes() {
+        // Build [MIN, MAX, 3661, NULL]
+        let arr = DurationSecondArray::from(vec![Some(i64::MIN), Some(i64::MAX), Some(3661), None]);
+        let array: ArrayRef = Arc::new(arr);
+
+        // Pretty formatting
+        let opts = FormatOptions::default().with_null("null");
+        let opts = opts.with_duration_format(DurationFormat::Pretty);
+        let pretty =
+            pretty_format_columns_with_options("pretty", std::slice::from_ref(&array), &opts)
+                .unwrap()
+                .to_string();
+
+        // Expected output
+        let expected_pretty = vec![
+            "+------------------------------+",
+            "| pretty                       |",
+            "+------------------------------+",
+            "| <invalid>                    |",
+            "| <invalid>                    |",
+            "| 0 days 1 hours 1 mins 1 secs |",
+            "| null                         |",
+            "+------------------------------+",
+        ];
+
+        let actual: Vec<&str> = pretty.lines().collect();
+        assert_eq!(expected_pretty, actual, "Actual result:\n{pretty}");
+
+        // ISO8601 formatting
+        let opts_iso = FormatOptions::default()
+            .with_null("null")
+            .with_duration_format(DurationFormat::ISO8601);
+        let iso = pretty_format_columns_with_options("iso", &[array], &opts_iso)
+            .unwrap()
+            .to_string();
+
+        // Expected output
+        let expected_iso = vec![
+            "+-----------+",
+            "| iso       |",
+            "+-----------+",
+            "| <invalid> |",
+            "| <invalid> |",
+            "| PT3661S   |",
+            "| null      |",
+            "+-----------+",
+        ];
+
+        let actual: Vec<&str> = iso.lines().collect();
+        assert_eq!(expected_iso, actual, "Actual result:\n{iso}");
+    }
 }
