@@ -17,14 +17,15 @@
 
 use arrow::array::{Array, ArrayRef, StringArray};
 use arrow::util::test_util::seedable_rng;
-use criterion::{criterion_group, criterion_main, Criterion};
+use criterion::{Criterion, criterion_group, criterion_main};
 use parquet_variant::{Variant, VariantBuilder};
-use parquet_variant_compute::variant_get::{variant_get, GetOptions};
-use parquet_variant_compute::{json_to_variant, VariantArray, VariantArrayBuilder};
-use rand::distr::Alphanumeric;
-use rand::rngs::StdRng;
+use parquet_variant_compute::{
+    GetOptions, VariantArray, VariantArrayBuilder, json_to_variant, variant_get,
+};
 use rand::Rng;
 use rand::SeedableRng;
+use rand::distr::Alphanumeric;
+use rand::rngs::StdRng;
 use std::fmt::Write;
 use std::sync::Arc;
 fn benchmark_batch_json_string_to_variant(c: &mut Criterion) {
@@ -84,7 +85,7 @@ fn benchmark_batch_json_string_to_variant(c: &mut Criterion) {
 
 pub fn variant_get_bench(c: &mut Criterion) {
     let variant_array = create_primitive_variant_array(8192);
-    let input: ArrayRef = Arc::new(variant_array);
+    let input = ArrayRef::from(variant_array);
 
     let options = GetOptions {
         path: vec![].into(),
