@@ -34,7 +34,7 @@ use arrow_schema::ArrowError;
 /// ```
 /// # use arrow::array::{Array, ArrayRef, Int64Array};
 /// # use parquet_variant::Variant;
-/// # use parquet_variant_compute::cast_to_variant::cast_to_variant;
+/// # use parquet_variant_compute::cast_to_variant;
 /// // input is an Int64Array, which will be cast to a VariantArray
 /// let input = Int64Array::from(vec![Some(1), None, Some(3)]);
 /// let result = cast_to_variant(&input).unwrap();
@@ -80,39 +80,38 @@ pub fn cast_to_variant(input: &dyn Array) -> Result<VariantArray, ArrowError> {
     cast_to_variant_with_options(input, &CastOptions::default())
 }
 
-// TODO do we need a cast_with_options to allow specifying conversion behavior,
-// e.g. how to handle overflows, whether to convert to Variant::Null or return
-// an error, etc. ?
-
 #[cfg(test)]
 mod tests {
     use super::*;
     use arrow::array::{
-        ArrayRef, BinaryArray, BooleanArray, Date32Array, Date64Array, Decimal128Array,
-        Decimal256Array, Decimal32Array, Decimal64Array, DictionaryArray, DurationMicrosecondArray,
-        DurationMillisecondArray, DurationNanosecondArray, DurationSecondArray,
-        FixedSizeBinaryBuilder, Float16Array, Float32Array, Float64Array, GenericByteBuilder,
-        GenericByteViewBuilder, Int16Array, Int32Array, Int64Array, Int8Array,
-        IntervalDayTimeArray, IntervalMonthDayNanoArray, IntervalYearMonthArray, LargeListArray,
-        LargeStringArray, ListArray, MapArray, NullArray, StringArray, StringRunBuilder,
-        StringViewArray, StructArray, Time32MillisecondArray, Time32SecondArray,
-        Time64MicrosecondArray, Time64NanosecondArray, TimestampMicrosecondArray,
-        TimestampMillisecondArray, TimestampNanosecondArray, TimestampSecondArray, UInt16Array,
-        UInt32Array, UInt64Array, UInt8Array, UnionArray,
+        ArrayRef, BinaryArray, BooleanArray, Date32Array, Date64Array, Decimal32Array,
+        Decimal64Array, Decimal128Array, Decimal256Array, DictionaryArray,
+        DurationMicrosecondArray, DurationMillisecondArray, DurationNanosecondArray,
+        DurationSecondArray, FixedSizeBinaryBuilder, FixedSizeListBuilder, Float16Array,
+        Float32Array, Float64Array, GenericByteBuilder, GenericByteViewBuilder, Int8Array,
+        Int16Array, Int32Array, Int64Array, IntervalDayTimeArray, IntervalMonthDayNanoArray,
+        IntervalYearMonthArray, LargeListArray, LargeListViewBuilder, LargeStringArray, ListArray,
+        ListViewBuilder, MapArray, NullArray, StringArray, StringRunBuilder, StringViewArray,
+        StructArray, Time32MillisecondArray, Time32SecondArray, Time64MicrosecondArray,
+        Time64NanosecondArray, TimestampMicrosecondArray, TimestampMillisecondArray,
+        TimestampNanosecondArray, TimestampSecondArray, UInt8Array, UInt16Array, UInt32Array,
+        UInt64Array, UnionArray,
     };
     use arrow::buffer::{NullBuffer, OffsetBuffer, ScalarBuffer};
     use arrow::datatypes::{
-        i256, BinaryType, BinaryViewType, Date32Type, Date64Type, Int32Type, Int64Type, Int8Type,
-        IntervalDayTime, IntervalMonthDayNano, LargeBinaryType,
+        BinaryType, BinaryViewType, Date32Type, Date64Type, Int8Type, Int32Type, Int64Type,
+        IntervalDayTime, IntervalMonthDayNano, LargeBinaryType, i256,
+    };
+    use arrow::temporal_conversions::timestamp_s_to_datetime;
+    use arrow_schema::{
+        DECIMAL32_MAX_PRECISION, DECIMAL64_MAX_PRECISION, DECIMAL128_MAX_PRECISION,
     };
     use arrow_schema::{DataType, Field, Fields, UnionFields};
-    use arrow_schema::{
-        DECIMAL128_MAX_PRECISION, DECIMAL32_MAX_PRECISION, DECIMAL64_MAX_PRECISION,
-    };
     use chrono::{DateTime, NaiveDate, NaiveTime};
     use half::f16;
     use parquet_variant::{
-        Variant, VariantBuilder, VariantDecimal16, VariantDecimal4, VariantDecimal8,
+        Variant, VariantBuilder, VariantBuilderExt, VariantDecimal4, VariantDecimal8,
+        VariantDecimal16,
     };
     use std::{sync::Arc, vec};
 
@@ -464,6 +463,20 @@ mod tests {
         )
     }
 
+    #[test]
+    fn test_cast_to_variant_decimal32_overflow_strict_mode() {
+        run_test_in_strict_mode(
+            Arc::new(
+                Decimal32Array::from(vec![Some(i32::MIN)])
+                    .with_precision_and_scale(DECIMAL32_MAX_PRECISION, 3)
+                    .unwrap(),
+            ),
+            Err(ArrowError::ComputeError(
+                "Failed to convert value at index 0: conversion failed".to_string(),
+            )),
+        );
+    }
+
     #[test]
     fn test_cast_to_variant_decimal64() {
         run_test(
@@ -554,6 +567,20 @@ mod tests {
         )
     }
 
+    #[test]
+    fn test_cast_to_variant_decimal64_overflow_strict_mode() {
+        run_test_in_strict_mode(
+            Arc::new(
+                Decimal64Array::from(vec![Some(i64::MAX)])
+                    .with_precision_and_scale(DECIMAL64_MAX_PRECISION, 3)
+                    .unwrap(),
+            ),
+            Err(ArrowError::ComputeError(
+                "Failed to convert value at index 0: conversion failed".to_string(),
+            )),
+        );
+    }
+
     #[test]
     fn test_cast_to_variant_decimal128() {
         run_test(
@@ -650,6 +677,22 @@ mod tests {
         )
     }
 
+    #[test]
+    fn test_cast_to_variant_decimal128_overflow_strict_mode() {
+        run_test_in_strict_mode(
+            Arc::new(
+                Decimal128Array::from(vec![Some(
+                    -max_unscaled_value!(128, DECIMAL128_MAX_PRECISION) - 1,
+                )])
+                .with_precision_and_scale(DECIMAL128_MAX_PRECISION, 3)
+                .unwrap(),
+            ),
+            Err(ArrowError::ComputeError(
+                "Failed to convert value at index 0: conversion failed".to_string(),
+            )),
+        );
+    }
+
     #[test]
     fn test_cast_to_variant_decimal256() {
         run_test(
@@ -766,6 +809,22 @@ mod tests {
         )
     }
 
+    #[test]
+    fn test_cast_to_variant_decimal256_overflow_strict_mode() {
+        run_test_in_strict_mode(
+            Arc::new(
+                Decimal256Array::from(vec![Some(i256::from_i128(
+                    max_unscaled_value!(128, DECIMAL128_MAX_PRECISION) + 1,
+                ))])
+                .with_precision_and_scale(DECIMAL128_MAX_PRECISION, 3)
+                .unwrap(),
+            ),
+            Err(ArrowError::ComputeError(
+                "Failed to convert value at index 0: conversion failed".to_string(),
+            )),
+        );
+    }
+
     #[test]
     fn test_cast_to_variant_timestamp() {
         let run_array_tests =
@@ -825,6 +884,40 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_cast_to_variant_timestamp_overflow_strict_mode() {
+        let ts_array = TimestampSecondArray::from(vec![Some(i64::MAX), Some(0), Some(1609459200)])
+            .with_timezone_opt(None::<&str>);
+
+        let values = Arc::new(ts_array);
+        run_test_in_strict_mode(
+            values,
+            Err(ArrowError::ComputeError(
+                "Failed to convert value at index 0: conversion failed".to_string(),
+            )),
+        );
+    }
+
+    #[test]
+    fn test_cast_to_variant_timestamp_overflow_non_strict_mode() {
+        let ts_array = TimestampSecondArray::from(vec![Some(i64::MAX), Some(0), Some(1609459200)])
+            .with_timezone_opt(None::<&str>);
+
+        let values = Arc::new(ts_array);
+        run_test(
+            values,
+            vec![
+                Some(Variant::Null), // Invalid timestamp becomes null
+                Some(Variant::TimestampNtzMicros(
+                    timestamp_s_to_datetime(0).unwrap(),
+                )),
+                Some(Variant::TimestampNtzMicros(
+                    timestamp_s_to_datetime(1609459200).unwrap(),
+                )),
+            ],
+        );
+    }
+
     #[test]
     fn test_cast_to_variant_date() {
         // Date32Array
@@ -864,6 +957,34 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_cast_to_variant_date64_strict_mode() {
+        let date64_values = Date64Array::from(vec![Some(i64::MAX), Some(0), Some(i64::MIN)]);
+
+        let values = Arc::new(date64_values);
+        run_test_in_strict_mode(
+            values,
+            Err(ArrowError::ComputeError(
+                "Failed to convert value at index 0: conversion failed".to_string(),
+            )),
+        );
+    }
+
+    #[test]
+    fn test_cast_to_variant_date64_non_strict_mode() {
+        let date64_values = Date64Array::from(vec![Some(i64::MAX), Some(0), Some(i64::MIN)]);
+
+        let values = Arc::new(date64_values);
+        run_test(
+            values,
+            vec![
+                Some(Variant::Null),
+                Some(Variant::Date(Date64Type::to_naive_date_opt(0).unwrap())),
+                Some(Variant::Null),
+            ],
+        );
+    }
+
     #[test]
     fn test_cast_to_variant_time32_second() {
         let array: Time32SecondArray = vec![Some(1), Some(86_399), None].into();
@@ -940,6 +1061,36 @@ mod tests {
         )
     }
 
+    #[test]
+    fn test_cast_to_variant_time32_strict_mode() {
+        let time32_array = Time32SecondArray::from(vec![Some(90000), Some(3600), Some(-1)]);
+
+        let values = Arc::new(time32_array);
+        run_test_in_strict_mode(
+            values,
+            Err(ArrowError::ComputeError(
+                "Failed to convert value at index 0: conversion failed".to_string(),
+            )),
+        );
+    }
+
+    #[test]
+    fn test_cast_to_variant_time32_non_strict_mode() {
+        let time32_array = Time32SecondArray::from(vec![Some(90000), Some(3600), Some(-1)]);
+
+        let values = Arc::new(time32_array);
+        run_test(
+            values,
+            vec![
+                Some(Variant::Null),
+                Some(Variant::Time(
+                    NaiveTime::from_num_seconds_from_midnight_opt(3600, 0).unwrap(),
+                )),
+                Some(Variant::Null),
+            ],
+        );
+    }
+
     #[test]
     fn test_cast_to_variant_duration_or_interval_errors() {
         let arrays: Vec<Box<dyn Array>> = vec![
@@ -1258,6 +1409,246 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_cast_to_variant_list_view() {
+        // Create a ListViewArray with some data
+        let mut builder = ListViewBuilder::new(Int32Array::builder(0));
+        builder.append_value(&Int32Array::from(vec![Some(0), None, Some(2)]));
+        builder.append_value(&Int32Array::from(vec![Some(3), Some(4)]));
+        builder.append_null();
+        builder.append_value(&Int32Array::from(vec![None, None]));
+        let list_view_array = builder.finish();
+
+        // Expected values
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_value(0i32);
+            list.append_null();
+            list.append_value(2i32);
+            list.finish();
+            builder.finish()
+        };
+        let variant0 = Variant::new(&metadata, &value);
+
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_value(3i32);
+            list.append_value(4i32);
+            list.finish();
+            builder.finish()
+        };
+        let variant1 = Variant::new(&metadata, &value);
+
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_null();
+            list.append_null();
+            list.finish();
+            builder.finish()
+        };
+        let variant3 = Variant::new(&metadata, &value);
+
+        run_test(
+            Arc::new(list_view_array),
+            vec![Some(variant0), Some(variant1), None, Some(variant3)],
+        );
+    }
+
+    #[test]
+    fn test_cast_to_variant_sliced_list_view() {
+        // Create a ListViewArray with some data
+        let mut builder = ListViewBuilder::new(Int32Array::builder(0));
+        builder.append_value(&Int32Array::from(vec![Some(0), Some(1), Some(2)]));
+        builder.append_value(&Int32Array::from(vec![Some(3), None]));
+        builder.append_null();
+        let list_view_array = builder.finish();
+
+        // Expected value for slice(1, 2) - should get the second and third elements
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_value(3i32);
+            list.append_null();
+            list.finish();
+            builder.finish()
+        };
+        let variant = Variant::new(&metadata, &value);
+
+        run_test(
+            Arc::new(list_view_array.slice(1, 2)),
+            vec![Some(variant), None],
+        );
+    }
+
+    #[test]
+    fn test_cast_to_variant_large_list_view() {
+        // Create a LargeListViewArray with some data
+        let mut builder = LargeListViewBuilder::new(Int64Array::builder(0));
+        builder.append_value(&Int64Array::from(vec![Some(0), None, Some(2)]));
+        builder.append_value(&Int64Array::from(vec![Some(3), Some(4)]));
+        builder.append_null();
+        builder.append_value(&Int64Array::from(vec![None, None]));
+        let large_list_view_array = builder.finish();
+
+        // Expected values
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_value(0i64);
+            list.append_null();
+            list.append_value(2i64);
+            list.finish();
+            builder.finish()
+        };
+        let variant0 = Variant::new(&metadata, &value);
+
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_value(3i64);
+            list.append_value(4i64);
+            list.finish();
+            builder.finish()
+        };
+        let variant1 = Variant::new(&metadata, &value);
+
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_null();
+            list.append_null();
+            list.finish();
+            builder.finish()
+        };
+        let variant3 = Variant::new(&metadata, &value);
+
+        run_test(
+            Arc::new(large_list_view_array),
+            vec![Some(variant0), Some(variant1), None, Some(variant3)],
+        );
+    }
+
+    #[test]
+    fn test_cast_to_variant_sliced_large_list_view() {
+        // Create a LargeListViewArray with some data
+        let mut builder = LargeListViewBuilder::new(Int64Array::builder(0));
+        builder.append_value(&Int64Array::from(vec![Some(0), Some(1), Some(2)]));
+        builder.append_value(&Int64Array::from(vec![Some(3), None]));
+        builder.append_null();
+        let large_list_view_array = builder.finish();
+
+        // Expected value for slice(1, 2) - should get the second and third elements
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_value(3i64);
+            list.append_null();
+            list.finish();
+            builder.finish()
+        };
+        let variant = Variant::new(&metadata, &value);
+
+        run_test(
+            Arc::new(large_list_view_array.slice(1, 2)),
+            vec![Some(variant), None],
+        );
+    }
+
+    #[test]
+    fn test_cast_to_variant_fixed_size_list() {
+        let mut builder = FixedSizeListBuilder::new(Int32Array::builder(0), 2);
+        builder.values().append_value(0);
+        builder.values().append_value(1);
+        builder.append(true); // First list: [0, 1]
+
+        builder.values().append_null();
+        builder.values().append_value(3);
+        builder.append(true); // Second list: [null, 3]
+
+        builder.values().append_value(4);
+        builder.values().append_null();
+        builder.append(false); // Third list: null
+
+        builder.values().append_nulls(2);
+        builder.append(true); // Last list: [null, null]
+
+        let fixed_size_list_array = builder.finish();
+
+        // Expected values
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_value(0i32);
+            list.append_value(1i32);
+            list.finish();
+            builder.finish()
+        };
+        let variant0 = Variant::new(&metadata, &value);
+
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_null();
+            list.append_value(3i32);
+            list.finish();
+            builder.finish()
+        };
+        let variant1 = Variant::new(&metadata, &value);
+
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_null();
+            list.append_null();
+            list.finish();
+            builder.finish()
+        };
+        let variant3 = Variant::new(&metadata, &value);
+
+        run_test(
+            Arc::new(fixed_size_list_array),
+            vec![Some(variant0), Some(variant1), None, Some(variant3)],
+        );
+    }
+
+    #[test]
+    fn test_cast_to_variant_sliced_fixed_size_list() {
+        // Create a FixedSizeListArray with size 2
+        let mut builder = FixedSizeListBuilder::new(Int64Array::builder(0), 2);
+        builder.values().append_value(0);
+        builder.values().append_value(1);
+        builder.append(true); // First list: [0, 1]
+
+        builder.values().append_null();
+        builder.values().append_value(3);
+        builder.append(true); // Second list: [null, 3]
+
+        builder.values().append_value(4);
+        builder.values().append_null();
+        builder.append(false); // Third list: null
+
+        let fixed_size_list_array = builder.finish();
+
+        // Expected value for slice(1, 2) - should get the second and third elements
+        let (metadata, value) = {
+            let mut builder = VariantBuilder::new();
+            let mut list = builder.new_list();
+            list.append_null();
+            list.append_value(3i64);
+            list.finish();
+            builder.finish()
+        };
+        let variant = Variant::new(&metadata, &value);
+
+        run_test(
+            Arc::new(fixed_size_list_array.slice(1, 2)),
+            vec![Some(variant), None],
+        );
+    }
+
     #[test]
     fn test_cast_to_variant_struct() {
         // Test a simple struct with two fields: id (int64) and age (int32)
@@ -1849,8 +2240,11 @@ mod tests {
     /// Converts the given `Array` to a `VariantArray` and tests the conversion
     /// against the expected values. It also tests the handling of nulls by
     /// setting one element to null and verifying the output.
-    fn run_test_with_options(values: ArrayRef, expected: Vec<Option<Variant>>, strict: bool) {
-        let options = CastOptions { strict };
+    fn run_test_with_options(
+        values: ArrayRef,
+        expected: Vec<Option<Variant>>,
+        options: CastOptions,
+    ) {
         let variant_array = cast_to_variant_with_options(&values, &options).unwrap();
         assert_eq!(variant_array.len(), expected.len());
         for (i, expected_value) in expected.iter().enumerate() {
@@ -1867,64 +2261,24 @@ mod tests {
     }
 
     fn run_test(values: ArrayRef, expected: Vec<Option<Variant>>) {
-        run_test_with_options(values, expected, true);
+        run_test_with_options(values, expected, CastOptions { strict: false });
     }
 
-    fn run_test_non_strict(values: ArrayRef, expected: Vec<Option<Variant>>) {
-        run_test_with_options(values, expected, false);
-    }
-
-    #[test]
-    fn test_cast_to_variant_non_strict_mode_date64() {
-        let date64_values = Date64Array::from(vec![Some(i64::MAX), Some(0), Some(i64::MIN)]);
-
-        let values = Arc::new(date64_values);
-        run_test_non_strict(
-            values,
-            vec![
-                None,
-                Some(Variant::Date(Date64Type::to_naive_date_opt(0).unwrap())),
-                None,
-            ],
-        );
-    }
-
-    #[test]
-    fn test_cast_to_variant_non_strict_mode_time32() {
-        let time32_array = Time32SecondArray::from(vec![Some(90000), Some(3600), Some(-1)]);
-
-        let values = Arc::new(time32_array);
-        run_test_non_strict(
-            values,
-            vec![
-                None,
-                Some(Variant::Time(
-                    NaiveTime::from_num_seconds_from_midnight_opt(3600, 0).unwrap(),
-                )),
-                None,
-            ],
-        );
-    }
-
-    #[test]
-    fn test_cast_to_variant_non_strict_mode_timestamp() {
-        use arrow::temporal_conversions::timestamp_s_to_datetime;
-
-        let ts_array = TimestampSecondArray::from(vec![Some(i64::MAX), Some(0), Some(1609459200)])
-            .with_timezone_opt(None::<&str>);
-
-        let values = Arc::new(ts_array);
-        run_test_non_strict(
-            values,
-            vec![
-                None, // Invalid timestamp becomes null
-                Some(Variant::TimestampNtzMicros(
-                    timestamp_s_to_datetime(0).unwrap(),
-                )),
-                Some(Variant::TimestampNtzMicros(
-                    timestamp_s_to_datetime(1609459200).unwrap(),
-                )),
-            ],
-        );
+    fn run_test_in_strict_mode(
+        values: ArrayRef,
+        expected: Result<Vec<Option<Variant>>, ArrowError>,
+    ) {
+        let options = CastOptions { strict: true };
+        match expected {
+            Ok(expected) => run_test_with_options(values, expected, options),
+            Err(_) => {
+                let result = cast_to_variant_with_options(values.as_ref(), &options);
+                assert!(result.is_err());
+                assert_eq!(
+                    result.unwrap_err().to_string(),
+                    expected.unwrap_err().to_string()
+                );
+            }
+        }
     }
 }
