@@ -95,10 +95,10 @@ impl<'a> From<Vec<VariantPathElement<'a>>> for VariantPath<'a> {
     }
 }
 
-/// Create from &str
+/// Create from &str with support for dot notation
 impl<'a> From<&'a str> for VariantPath<'a> {
     fn from(path: &'a str) -> Self {
-        VariantPath::new(vec![path.into()])
+        VariantPath::new(path.split('.').map(Into::into).collect())
     }
 }
 
@@ -109,6 +109,12 @@ impl<'a> From<usize> for VariantPath<'a> {
     }
 }
 
+impl<'a> From<&[VariantPathElement<'a>]> for VariantPath<'a> {
+    fn from(elements: &[VariantPathElement<'a>]) -> Self {
+        VariantPath::new(elements.to_vec())
+    }
+}
+
 /// Create from iter
 impl<'a> FromIterator<VariantPathElement<'a>> for VariantPath<'a> {
     fn from_iter<T: IntoIterator<Item = VariantPathElement<'a>>>(iter: T) -> Self {
