@@ -17,14 +17,12 @@
 
 //! Converting Parquet schema <--> Arrow schema: [`ArrowSchemaConverter`] and [parquet_to_arrow_schema]
 
-use base64::prelude::BASE64_STANDARD;
 use base64::Engine;
+use base64::prelude::BASE64_STANDARD;
 use std::collections::HashMap;
 use std::sync::Arc;
 
 use arrow_ipc::writer;
-#[cfg(feature = "arrow_canonical_extension_types")]
-use arrow_schema::extension::{Json, Uuid};
 use arrow_schema::{DataType, Field, Fields, Schema, TimeUnit};
 
 use crate::basic::{
@@ -35,12 +33,16 @@ use crate::file::{metadata::KeyValue, properties::WriterProperties};
 use crate::schema::types::{ColumnDescriptor, SchemaDescriptor, Type};
 
 mod complex;
+mod extension;
 mod primitive;
 
-use crate::arrow::ProjectionMask;
-pub(crate) use complex::{ParquetField, ParquetFieldType};
-
 use super::PARQUET_FIELD_ID_META_KEY;
+use crate::arrow::ProjectionMask;
+use crate::arrow::schema::extension::{
+    has_extension_type, logical_type_for_fixed_size_binary, logical_type_for_string,
+    logical_type_for_struct, try_add_extension_type,
+};
+pub(crate) use complex::{ParquetField, ParquetFieldType};
 
 /// Convert Parquet schema to Arrow schema including optional metadata
 ///
@@ -63,7 +65,11 @@ pub fn parquet_to_arrow_schema_by_columns(
     Ok(parquet_to_arrow_schema_and_fields(parquet_schema, mask, key_value_metadata)?.0)
 }
 
-/// Extracts the arrow metadata
+/// Determines the Arrow Schema from a Parquet schema
+///
+/// Looks for an Arrow schema metadata "hint" (see
+/// [`parquet_to_arrow_field_levels`]), and uses it if present to ensure
+/// lossless round trips.
 pub(crate) fn parquet_to_arrow_schema_and_fields(
     parquet_schema: &SchemaDescriptor,
     mask: ProjectionMask,
@@ -370,11 +376,7 @@ fn parse_key_value_metadata(
                 })
                 .collect();
 
-            if map.is_empty() {
-                None
-            } else {
-                Some(map)
-            }
+            if map.is_empty() { None } else { Some(map) }
         }
         None => None,
     }
@@ -385,31 +387,27 @@ pub fn parquet_to_arrow_field(parquet_column: &ColumnDescriptor) -> Result<Field
     let field = complex::convert_type(&parquet_column.self_type_ptr())?;
     let mut ret = Field::new(parquet_column.name(), field.arrow_type, field.nullable);
 
-    let basic_info = parquet_column.self_type().get_basic_info();
-    let mut meta = HashMap::with_capacity(if cfg!(feature = "arrow_canonical_extension_types") {
-        2
-    } else {
-        1
-    });
+    let parquet_type = parquet_column.self_type();
+    let basic_info = parquet_type.get_basic_info();
+
+    let mut hash_map_size = 0;
     if basic_info.has_id() {
-        meta.insert(
+        hash_map_size += 1;
+    }
+    if has_extension_type(parquet_type) {
+        hash_map_size += 1;
+    }
+    if hash_map_size == 0 {
+        return Ok(ret);
+    }
+    ret.set_metadata(HashMap::with_capacity(hash_map_size));
+    if basic_info.has_id() {
+        ret.metadata_mut().insert(
             PARQUET_FIELD_ID_META_KEY.to_string(),
             basic_info.id().to_string(),
         );
     }
-    #[cfg(feature = "arrow_canonical_extension_types")]
-    if let Some(logical_type) = basic_info.logical_type() {
-        match logical_type {
-            LogicalType::Uuid => ret.try_with_extension_type(Uuid)?,
-            LogicalType::Json => ret.try_with_extension_type(Json::default())?,
-            _ => {}
-        }
-    }
-    if !meta.is_empty() {
-        ret.set_metadata(meta);
-    }
-
-    Ok(ret)
+    try_add_extension_type(ret, parquet_column.self_type())
 }
 
 pub fn decimal_length_from_precision(precision: u8) -> usize {
@@ -532,9 +530,9 @@ fn arrow_to_parquet_type(field: &Field, coerce_types: bool) -> Result<Type> {
                     is_adjusted_to_u_t_c: matches!(tz, Some(z) if !z.as_ref().is_empty()),
                     unit: match time_unit {
                         TimeUnit::Second => unreachable!(),
-                        TimeUnit::Millisecond => ParquetTimeUnit::MILLIS(Default::default()),
-                        TimeUnit::Microsecond => ParquetTimeUnit::MICROS(Default::default()),
-                        TimeUnit::Nanosecond => ParquetTimeUnit::NANOS(Default::default()),
+                        TimeUnit::Millisecond => ParquetTimeUnit::MILLIS,
+                        TimeUnit::Microsecond => ParquetTimeUnit::MICROS,
+                        TimeUnit::Nanosecond => ParquetTimeUnit::NANOS,
                     },
                 }))
                 .with_repetition(repetition)
@@ -571,7 +569,7 @@ fn arrow_to_parquet_type(field: &Field, coerce_types: bool) -> Result<Type> {
             .with_logical_type(Some(LogicalType::Time {
                 is_adjusted_to_u_t_c: field.metadata().contains_key("adjusted_to_utc"),
                 unit: match unit {
-                    TimeUnit::Millisecond => ParquetTimeUnit::MILLIS(Default::default()),
+                    TimeUnit::Millisecond => ParquetTimeUnit::MILLIS,
                     u => unreachable!("Invalid unit for Time32: {:?}", u),
                 },
             }))
@@ -582,8 +580,8 @@ fn arrow_to_parquet_type(field: &Field, coerce_types: bool) -> Result<Type> {
             .with_logical_type(Some(LogicalType::Time {
                 is_adjusted_to_u_t_c: field.metadata().contains_key("adjusted_to_utc"),
                 unit: match unit {
-                    TimeUnit::Microsecond => ParquetTimeUnit::MICROS(Default::default()),
-                    TimeUnit::Nanosecond => ParquetTimeUnit::NANOS(Default::default()),
+                    TimeUnit::Microsecond => ParquetTimeUnit::MICROS,
+                    TimeUnit::Nanosecond => ParquetTimeUnit::NANOS,
                     u => unreachable!("Invalid unit for Time64: {:?}", u),
                 },
             }))
@@ -613,16 +611,7 @@ fn arrow_to_parquet_type(field: &Field, coerce_types: bool) -> Result<Type> {
                 .with_repetition(repetition)
                 .with_id(id)
                 .with_length(*length)
-                .with_logical_type(
-                    #[cfg(feature = "arrow_canonical_extension_types")]
-                    // If set, map arrow uuid extension type to parquet uuid logical type.
-                    field
-                        .try_extension_type::<Uuid>()
-                        .ok()
-                        .map(|_| LogicalType::Uuid),
-                    #[cfg(not(feature = "arrow_canonical_extension_types"))]
-                    None,
-                )
+                .with_logical_type(logical_type_for_fixed_size_binary(field))
                 .build()
         }
         DataType::BinaryView => Type::primitive_type_builder(name, PhysicalType::BYTE_ARRAY)
@@ -659,35 +648,13 @@ fn arrow_to_parquet_type(field: &Field, coerce_types: bool) -> Result<Type> {
         }
         DataType::Utf8 | DataType::LargeUtf8 => {
             Type::primitive_type_builder(name, PhysicalType::BYTE_ARRAY)
-                .with_logical_type({
-                    #[cfg(feature = "arrow_canonical_extension_types")]
-                    {
-                        // Use the Json logical type if the canonical Json
-                        // extension type is set on this field.
-                        field
-                            .try_extension_type::<Json>()
-                            .map_or(Some(LogicalType::String), |_| Some(LogicalType::Json))
-                    }
-                    #[cfg(not(feature = "arrow_canonical_extension_types"))]
-                    Some(LogicalType::String)
-                })
+                .with_logical_type(logical_type_for_string(field))
                 .with_repetition(repetition)
                 .with_id(id)
                 .build()
         }
         DataType::Utf8View => Type::primitive_type_builder(name, PhysicalType::BYTE_ARRAY)
-            .with_logical_type({
-                #[cfg(feature = "arrow_canonical_extension_types")]
-                {
-                    // Use the Json logical type if the canonical Json
-                    // extension type is set on this field.
-                    field
-                        .try_extension_type::<Json>()
-                        .map_or(Some(LogicalType::String), |_| Some(LogicalType::Json))
-                }
-                #[cfg(not(feature = "arrow_canonical_extension_types"))]
-                Some(LogicalType::String)
-            })
+            .with_logical_type(logical_type_for_string(field))
             .with_repetition(repetition)
             .with_id(id)
             .build(),
@@ -728,6 +695,7 @@ fn arrow_to_parquet_type(field: &Field, coerce_types: bool) -> Result<Type> {
                 .with_fields(fields)
                 .with_repetition(repetition)
                 .with_id(id)
+                .with_logical_type(logical_type_for_struct(field))
                 .build()
         }
         DataType::Map(field, _) => {
@@ -769,7 +737,7 @@ fn arrow_to_parquet_type(field: &Field, coerce_types: bool) -> Result<Type> {
             }
         }
         DataType::Union(_, _) => unimplemented!("See ARROW-8817."),
-        DataType::Dictionary(_, ref value) => {
+        DataType::Dictionary(_, value) => {
             // Dictionary encoding not handled at the schema level
             let dict_field = field.clone().with_data_type(value.as_ref().clone());
             arrow_to_parquet_type(&dict_field, coerce_types)
@@ -791,15 +759,14 @@ mod tests {
 
     use std::{collections::HashMap, sync::Arc};
 
-    use arrow::datatypes::{DataType, Field, IntervalUnit, TimeUnit};
-
     use crate::arrow::PARQUET_FIELD_ID_META_KEY;
     use crate::file::metadata::KeyValue;
     use crate::file::reader::FileReader;
     use crate::{
-        arrow::{arrow_reader::ParquetRecordBatchReaderBuilder, ArrowWriter},
+        arrow::{ArrowWriter, arrow_reader::ParquetRecordBatchReaderBuilder},
         schema::{parser::parse_message_type, types::SchemaDescriptor},
     };
+    use arrow::datatypes::{DataType, Field, IntervalUnit, TimeUnit};
 
     #[test]
     fn test_flat_primitives() {
@@ -838,12 +805,26 @@ mod tests {
             Field::new("float16", DataType::Float16, true),
             Field::new("string", DataType::Utf8, true),
             Field::new("string_2", DataType::Utf8, true),
-            Field::new("json", DataType::Utf8, true),
+            json_field(),
         ]);
 
         assert_eq!(&arrow_fields, converted_arrow_schema.fields());
     }
 
+    /// Return the expected Field for a Parquet column annotated with
+    /// the JSON logical type.
+    fn json_field() -> Field {
+        #[cfg(feature = "arrow_canonical_extension_types")]
+        {
+            Field::new("json", DataType::Utf8, true)
+                .with_extension_type(arrow_schema::extension::Json::default())
+        }
+        #[cfg(not(feature = "arrow_canonical_extension_types"))]
+        {
+            Field::new("json", DataType::Utf8, true)
+        }
+    }
+
     #[test]
     fn test_decimal_fields() {
         let message_type = "
@@ -2227,12 +2208,10 @@ mod tests {
     #[test]
     #[cfg(feature = "arrow_canonical_extension_types")]
     fn arrow_uuid_to_parquet_uuid() -> Result<()> {
-        let arrow_schema = Schema::new(vec![Field::new(
-            "uuid",
-            DataType::FixedSizeBinary(16),
-            false,
-        )
-        .with_extension_type(Uuid)]);
+        use arrow_schema::extension::Uuid;
+        let arrow_schema = Schema::new(vec![
+            Field::new("uuid", DataType::FixedSizeBinary(16), false).with_extension_type(Uuid),
+        ]);
 
         let parquet_schema = ArrowSchemaConverter::new().convert(&arrow_schema)?;
 
@@ -2241,9 +2220,8 @@ mod tests {
             Some(LogicalType::Uuid)
         );
 
-        // TODO: roundtrip
-        // let arrow_schema = parquet_to_arrow_schema(&parquet_schema, None)?;
-        // assert_eq!(arrow_schema.field(0).try_extension_type::<Uuid>()?, Uuid);
+        let arrow_schema = parquet_to_arrow_schema(&parquet_schema, None)?;
+        assert_eq!(arrow_schema.field(0).try_extension_type::<Uuid>()?, Uuid);
 
         Ok(())
     }
@@ -2251,8 +2229,9 @@ mod tests {
     #[test]
     #[cfg(feature = "arrow_canonical_extension_types")]
     fn arrow_json_to_parquet_json() -> Result<()> {
+        use arrow_schema::extension::Json;
         let arrow_schema = Schema::new(vec![
-            Field::new("json", DataType::Utf8, false).with_extension_type(Json::default())
+            Field::new("json", DataType::Utf8, false).with_extension_type(Json::default()),
         ]);
 
         let parquet_schema = ArrowSchemaConverter::new().convert(&arrow_schema)?;
@@ -2262,13 +2241,11 @@ mod tests {
             Some(LogicalType::Json)
         );
 
-        // TODO: roundtrip
-        // https://github.com/apache/arrow-rs/issues/7063
-        // let arrow_schema = parquet_to_arrow_schema(&parquet_schema, None)?;
-        // assert_eq!(
-        //     arrow_schema.field(0).try_extension_type::<Json>()?,
-        //     Json::default()
-        // );
+        let arrow_schema = parquet_to_arrow_schema(&parquet_schema, None)?;
+        assert_eq!(
+            arrow_schema.field(0).try_extension_type::<Json>()?,
+            Json::default()
+        );
 
         Ok(())
     }
