@@ -17,6 +17,10 @@
 
 //! Binary that prints the physical layout of a parquet file
 //!
+//! NOTE: due to this binary's use of the deprecated [`parquet::format`] module, it
+//! will no longer be maintained, and will likely be removed in the future.
+//! Alternatives to this include [`parquet-cli`] and [`parquet-viewer`].
+//!
 //! # Install
 //!
 //! `parquet-layout` can be installed using `cargo`:
@@ -32,17 +36,22 @@
 //! ```
 //! cargo run --features=cli --bin parquet-layout XYZ.parquet
 //! ```
+//!
+//! [`parquet-cli`]: https://github.com/apache/parquet-java/tree/master/parquet-cli
+//! [`parquet-viewer`]: https://github.com/xiangpenghao/parquet-viewer
 
 use std::fs::File;
 use std::io::Read;
 
 use clap::Parser;
+use parquet::file::metadata::ParquetMetaDataReader;
 use serde::Serialize;
 use thrift::protocol::TCompactInputProtocol;
 
-use parquet::basic::{Compression, Encoding};
+use parquet::basic::Compression;
 use parquet::errors::Result;
 use parquet::file::reader::ChunkReader;
+#[allow(deprecated)]
 use parquet::format::PageHeader;
 use parquet::thrift::TSerializable;
 
@@ -78,8 +87,9 @@ struct Page {
     num_values: i32,
 }
 
+#[allow(deprecated)]
 fn do_layout<C: ChunkReader>(reader: &C) -> Result<ParquetFile> {
-    let metadata = parquet::file::footer::parse_metadata(reader)?;
+    let metadata = ParquetMetaDataReader::new().parse_and_finish(reader)?;
     let schema = metadata.file_metadata().schema_descr();
 
     let row_groups = (0..metadata.num_row_groups())
@@ -104,7 +114,7 @@ fn do_layout<C: ChunkReader>(reader: &C) -> Result<ParquetFile> {
                         if let Some(dictionary) = header.dictionary_page_header {
                             pages.push(Page {
                                 compression,
-                                encoding: encoding(dictionary.encoding),
+                                encoding: encoding(dictionary.encoding.0),
                                 page_type: "dictionary",
                                 offset: start,
                                 compressed_bytes: header.compressed_page_size,
@@ -115,7 +125,7 @@ fn do_layout<C: ChunkReader>(reader: &C) -> Result<ParquetFile> {
                         } else if let Some(data_page) = header.data_page_header {
                             pages.push(Page {
                                 compression,
-                                encoding: encoding(data_page.encoding),
+                                encoding: encoding(data_page.encoding.0),
                                 page_type: "data_page_v1",
                                 offset: start,
                                 compressed_bytes: header.compressed_page_size,
@@ -128,7 +138,7 @@ fn do_layout<C: ChunkReader>(reader: &C) -> Result<ParquetFile> {
 
                             pages.push(Page {
                                 compression: compression.filter(|_| is_compressed),
-                                encoding: encoding(data_page.encoding),
+                                encoding: encoding(data_page.encoding.0),
                                 page_type: "data_page_v2",
                                 offset: start,
                                 compressed_bytes: header.compressed_page_size,
@@ -162,6 +172,7 @@ fn do_layout<C: ChunkReader>(reader: &C) -> Result<ParquetFile> {
 
 /// Reads the page header at `offset` from `reader`, returning
 /// both the `PageHeader` and its length in bytes
+#[allow(deprecated)]
 fn read_page_header<C: ChunkReader>(reader: &C, offset: u64) -> Result<(usize, PageHeader)> {
     struct TrackedRead<R>(R, usize);
 
@@ -195,18 +206,19 @@ fn compression(compression: Compression) -> Option<&'static str> {
 }
 
 /// Returns a string representation for a given encoding
-fn encoding(encoding: parquet::format::Encoding) -> &'static str {
-    match Encoding::try_from(encoding) {
-        Ok(Encoding::PLAIN) => "plain",
-        Ok(Encoding::PLAIN_DICTIONARY) => "plain_dictionary",
-        Ok(Encoding::RLE) => "rle",
-        Ok(Encoding::BIT_PACKED) => "bit_packed",
-        Ok(Encoding::DELTA_BINARY_PACKED) => "delta_binary_packed",
-        Ok(Encoding::DELTA_LENGTH_BYTE_ARRAY) => "delta_length_byte_array",
-        Ok(Encoding::DELTA_BYTE_ARRAY) => "delta_byte_array",
-        Ok(Encoding::RLE_DICTIONARY) => "rle_dictionary",
-        Ok(Encoding::BYTE_STREAM_SPLIT) => "byte_stream_split",
-        Err(_) => "unknown",
+fn encoding(encoding: i32) -> &'static str {
+    match encoding {
+        0 => "plain",
+        2 => "plain_dictionary",
+        3 => "rle",
+        #[allow(deprecated)]
+        4 => "bit_packed",
+        5 => "delta_binary_packed",
+        6 => "delta_length_byte_array",
+        7 => "delta_byte_array",
+        8 => "rle_dictionary",
+        9 => "byte_stream_split",
+        _ => "unknown",
     }
 }
 
