@@ -19,5 +19,6 @@
 //!
 //! [Column Index]: https://github.com/apache/parquet-format/blob/master/PageIndex.md
 
-pub mod index;
+pub mod column_index;
 pub mod index_reader;
+pub mod offset_index;
